/*
 Copyright 2015 Adelmo De Santis
 * This program is released under the GPLv3 License.
 * 
 *  This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/*
 * Release notes:
 * Version 1.0  Released on 17th July 2015 (Internal)
 *  Si570 programming feature, 2 leds nothing more, nothing less 
 * Version 1.1  Released on 24th July 2015 (Internal)
 *  Added capability of DFS-1301 Version C and D Programming
 *  Support For 12648MHz and 12672 MHz
 * Version 1.2  Released on 27th July 2015 (Internal)
 *  Added support for 432MHz and 1296MHz IFs;
 *  Program function reorganized
 *  Added test functionality.
 *  Added support for 16bit long N register.
 * Version 1.3 Released on 28th July 2015 (Internal)
 *  Revised code on program function
 *  Revided code on  test-dfs function
 * Version 1.4 Released on 31st July 2015 (Internal)
 *  Added support for DFS-1201 Rev.D
 * Version 1.5 Released on 28th August 2015 (Internal)
 *  Added support for DFS-1101 Rev.C
 * Version 1.6 Released on 18th September 2015 (Internal)
 *  Added support for ILCDFSL-1201 Rev C
 * Version 1.7 Released on xx November 2015 (Internal)
 *  Added PARTIAL support for ILCDFSL-1301 REV C
 * Version 1.8 Released on 30 Novembre 2015 (Internal)
 *  Added support for PicBoard V3.1
 * 
 */

/*
 * To do List:
 */

#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#include <time.h>
#include <pic18f25k50.h>
#include <i2c.h>
#include <math.h>
#include <stdio.h>

#pragma config FOSC = INTOSCIO     /* Oscillator Selection (Internal oscillator)*/
#pragma config CPUDIV = NOCLKDIV   /* CPU uses system clock (no divide)*/
#pragma config PCLKEN = ON         /* Primary Oscillator Shutdown (Primary oscillator enabled)*/
#pragma config CFGPLLEN = OFF      /* Pll not enabled in INTOSCIO mode*/
#pragma config FCMEN =  OFF        /* Allows the device to continue operating even if the external oscillator should fail*/
#pragma config IESO =  OFF         /* If FCMEN is on then IESO must be on as well.*/
#pragma config BOREN = NOSLP       
#pragma config BORV = 250
#pragma config WDTEN = OFF
#pragma config MCLRE = ON
#pragma config DEBUG = ON

/* Hardware dependent configuration.*/
/* HARDWARE CONFIGURATION 
 * RA0  -   Switch per selezione DEBUG mode    
 * RA1  -   Dip Switch 1
 * RA2  -   Dip Switch 2
 * RA3  -   Dip Switch 3
 * RA4  -   Dip Switch 4
 * RA5  -   Dip Switch 5
 * RA6  -   LED 2
 * RA7  -   LED 1
 
 * RB0  -   I2C sda
 * RB1  -   I2C SCL
 * RB2  -   LOCK
 * RB3  -   ENABLE
 * RB4  -   CLOCK
 * RB5  -   DATA
 * RB6  -   PGC
 * RB7  -   PGD
  
 * RC0  -   Debug Data
 * RC1  -   Debug Clock
 * RC2  -   Dip Switch 6
 * RC6  -   Dip Switch 7
 * RC7  -   Dip Switch 8
 
 */

#define DSW1        PORTAbits.RA0   //board v1
#define DSW2        PORTAbits.RA1   //board v1
#define DSW3        PORTAbits.RA2   //board v3
#define DSW4        PORTAbits.RA3   //board v3
#define DSW5        PORTAbits.RA4   //board v3
#define DSW6        PORTAbits.RA5   //board v3
#define LED2        PORTAbits.RA6   //board v1
#define LED1        PORTAbits.RA7   //board v1

#define DFS_LOCK    PORTBbits.RB2   //board v1
#define DFS_ENABLE  PORTBbits.RB3   //board v1
#define DFS_CLOCK   PORTBbits.RB4   //board v1
#define DFS_DATA    PORTBbits.RB5   //board v1

#define DBG_DATA    PORTCbits.RC0
#define DBG_CLOCK   PORTCbits.RC1
#define DSW7        PORTCbits.RC2
#define DSW8        PORTCbits.RC6
#define DSW9        PORTCbits.RC7

/* Si570 address. In this case it is left shifted by 1 bit.*/
#define siadd       0xAA
/* system clock frequency */
#define _XTAL_FREQ          16000000

/* Mathematica constants */
#define POW_2_16	65536.0				/* Power of 2 */
#define POW_2_24	16777216.0
#define POW_2_28	268435456.0

/* Chip parameters - come from datasheet */
#define FOUT0_START_UP	56.319796		/* Startup frequency (Measured)	*/
#define FDCO_MAX	    5670			/* Max DCO Frequency 	*/
#define FDCO_MIN        4850			/* Min DCO Frequency	*/
#define MinRange        10.0			/* Max Output Frequency */
#define MaxRange        800.0			/* Min Output Frequency */
#define MultHi          1.0035			/* */
#define MultLo          0.9965			/* */
//#define FXTAL           114.300895      /* */

/* This is the most important variable. It stores the contents of si570 registers
 * It is used both while reading the configuration from the device, and 
 * while programming it. 
 */
unsigned int REG[6];
unsigned int CFG;

/* Variables to store some important values; */
unsigned int INITIAL_HSDIV;				/* Value read from Si570 at startup */ 
unsigned int READ_HSDIV;				/* Value read from Si570 at work time */ 
unsigned int INITIAL_N1;				/* Value read from Si570 at startup */ 
unsigned int READ_N1;					/* Value read from Si570 at work time */ 

unsigned long int INITIAL_RFREQ_LONG_INT;	/* Long Int conversion of RFREQ at startup */
unsigned long int INITIAL_RFREQ_LONG_FRAC;	/* Long Int conversion of RFREQ at startup */
unsigned long int READ_RFREQ_LONG;		/* Long Int conversion of RFREQ at work time */
unsigned long int FRAC_BITS;			/* Only decimal part of RFREQ */

unsigned int status;


/* Some big floats */
float RFREQ;	/* RFREQ register */
float FXTAL;	/* Value of Si570's FXTAL - calculated */

/* Worktime Variables for Frequency setup */
float upbou;	/* Upper frequency boud - Used to discriminate between small and large frequency changes */
float lowbou;	/* Lower frequency boud - Used to discriminate between small and large frequency changes */

/* HSDIV can take only discrete values */
int HSDIV[]={11,9,7,6,5,4};
/* Is set to 1 if a valid combination of N1 and HSDIV is found  */
bit ValidCombo;//

void delay_long(unsigned int usec){
    /* This function generates some delays by using the PicMicro timer and
     * interrupt systems.
     * limit is determined as follow: limit=(16000000*us)/1000000
     * Let's recall that OSCCON is 0xF2 -> 1111 010
     * 1    device enter idle mode  on sleep
     * 111  HFINTOSC (16MHz)
     * 0    device is running from internal osc
     * 0    stable frequency
     * 1x   internal oscillator
     */
    unsigned int limit;
    unsigned int LowNibble,HighNibble;
   switch(usec){
       case 0:      limit=32468;            //82usec
                    T0CONbits.TMR0ON  = 0;
                    T0CONbits.PSA     = 1;
                    INTCONbits.TMR0IF = 0;
                    break;
        case 1:     limit=64936;            //164usec
                    T0CONbits.TMR0ON  = 0;
                    T0CONbits.PSA     = 1;
                    INTCONbits.TMR0IF = 0;
                    break;
        case 2:     limit=25000;            //10msec
                    T0CONbits.TMR0ON  = 0;
                    T0CONbits.PSA     = 1;
                    INTCONbits.TMR0IF = 0;
                    break;
        case 3:     T0CONbits.TMR0ON = 0;   //200ms
                    T0CONbits.PSA   = 0;    // A prescaler is assigned for Timer0
                    T0CONbits.T0PS2 = 1;    // Use a 1:256 prescaler
                    T0CONbits.T0PS1 = 1;
                    T0CONbits.T0PS0 = 1;
                    INTCONbits.TMR0IF = 0;
                    limit=62500;
                    break;
        case 4:     T0CONbits.TMR0ON = 0;   //1 s
                    T0CONbits.PSA   = 0;    // A prescaler is assigned for Timer0
                    T0CONbits.T0PS2 = 1;    // Use a 1:256 prescaler
                    T0CONbits.T0PS1 = 1;
                    T0CONbits.T0PS0 = 1;
                    INTCONbits.TMR0IF = 0;
                    limit=49000;
                    break;
        case 5:     limit=13000;            //6msec
                    T0CONbits.TMR0ON  = 0;
                    T0CONbits.PSA     = 1;
                    INTCONbits.TMR0IF = 0;
                    break;
        case 6:     limit=64936;            //100 usec
                    T0CONbits.TMR0ON  = 0;
                    T0CONbits.PSA     = 1;
                    INTCONbits.TMR0IF = 0;
                    break;
    }
    LowNibble=(limit)&0xFF;
    HighNibble=(limit&0xFF00)>>8;

    INTCONbits.TMR0IF = 0;

    /* Coupled registers in order to reach 16bit depth.*/
    TMR0H=HighNibble; 
    TMR0L=LowNibble;

    T0CONbits.TMR0ON = 1;

    while(!INTCONbits.TMR0IF)      /* wait until Timer0 overflows - polling */
    { }
    INTCONbits.TMR0IF = 0;
}

void initHardware() {
    /* This function performs some PIC Micro setup routines. It programs ports baheviour 
       and set up oscillators and interrupts.
     */
    OSCCON = 0xF0;  /* 1111 0010 Internal oscillator at 16 MHz 
                     * System Clock is provided from internal 
                     * oscillator block */

    ANSELA = 0x00;
    TRISA  = 0x3F;
    ANSELB = 0x04;  /* Configure RB2 as analog input: TRIS=1 - ANSEL=1 RB0=SDA RB1=SCL */
    TRISB  = 0x07;  /* All outputs! allow for i2c */
    ANSELC = 0x00;
    TRISC  = 0xC4;

    /* Oscillator and interrupt configuration*/
    T0CONbits.TMR0ON = 0;   /* Stop the timer*/
    T0CONbits.T08BIT = 0;   /* Run in 16-bit mode*/
    T0CONbits.T0CS =   0;   /* Use system clock to increment timer FOSC/4*/
    T0CONbits.PSA = 1;      /* A prescaler is assigned for Timer0 pay attention: 0-> assigned 1-> not assigned*/
    T0CONbits.T0PS2 = 1;    /* Use a 1:256 prescaler*/
    T0CONbits.T0PS1 = 1;
    T0CONbits.T0PS0 = 1;
    
    /* Interrupt configuration */
    INTCONbits.TMR0IF = 0;  /* Clear Timer0 interrupt flag */
    PIR1bits.TMR1IF	  = 0;	/* Clear Timer1 interrupt flag */
    
    /* I2C Stuff is here */
    SSP1ADD  = 0x27;                /* 0x27-> 39d so applaying Fi2c=[((SSP1ADD+1)*4)/Fosc] it gives: clock = 100 kHz */
    SSP1STAT = 0x80;;               /* slew rate control disabled */
    SSP1CON1  = 0b00111000;         /* module enabled, I2C master */
    SSP1CON2  = 0;                  
    SSP1CON3  = 0b01100000;

 }//init hardware

unsigned int read_i2c(unsigned int dev_address, unsigned int read_address){
    /* this function performs I2C bus reading in  byte oriented way. Code was 
     * optimized at the oscilloscope. There are several way to improve it. In this
     * formulation it works. Please notice the commented IdleI2C commands. 
     * 
     * During code development a problem arose: after the transmission of the third byte 
     * transmission was stopped and slave held the bus stuck. It seemed to me an hardware
     * problem, but it was mitigated with the Idle removal.
    */
    unsigned int dataout;
    
    dev_address=dev_address<<1;
    IdleI2C();                         // Wait until the bus is idle
    StartI2C();                        // Send START condition
    IdleI2C();                         // Wait for the end of the START condition
    WriteI2C(dev_address | 0x00);      // Send address with R/W set for read
    //IdleI2C();                       // Wait for ACK
    WriteI2C(read_address);
    //IdleI2C();

    RestartI2C();
    IdleI2C();
    WriteI2C(dev_address | 0x01);
    //IdleI2C();                       // Wait for ACK
    dataout = ReadI2C();               // Read first byte of data
    //AckI2C();                        // Send ACK
    StopI2C();                         // Hang up, send STOP condition
    IdleI2C();
    
    return dataout;
}

void write_i2c(unsigned int address_dev, unsigned int radd, unsigned int data){
    /* Very simple write-on-the-bus function.*/
   IdleI2C();
   StartI2C();
   IdleI2C();
   WriteI2C(address_dev);
   IdleI2C();
   WriteI2C(radd);
   IdleI2C();
   WriteI2C(data);
   StopI2C();
}

void dco_status(unsigned int status){
    /* This function performs the freeze and unfreeze of the DCO.*/
    unsigned int reg137;
    
    switch(status){
        case '1' :
            reg137=read_i2c(0xAA,0x89);
            delay_long(2);
            /* set bit number 4 to 1 to freeze DCO */
            reg137=reg137|0x10;
            write_i2c(0xAA,0x89,reg137);
            delay_long(2);
            break;
        case '2':
            reg137=read_i2c(0xAA,0x89);
            delay_long(2);
            /* set bit number 4 to 0 to unfreeze DCO */
            reg137=reg137|0xef;
            write_i2c(0xAA,0x89,reg137);
            delay_long(2);
            break;
            
    }
    
    
}

unsigned int SetBits(unsigned int original, unsigned int reset_mask, unsigned int new_val) {
return((original & reset_mask)| new_val);

}//SetBits

void FreqProg(float FOUT){
/* When called it performs some important action in order to define register contents and program Si570 frequency
 * It takes the desidered frequency as input and calculates best values for n1 and hsdiv. Output is REG array 
 * More details can be found on the application note AN334 of Silicon Labs. Notice that this AN is included in this
 * zip file in the integrazioni folder.
 * INPUT 	void
 * OUTPUT 	void */

unsigned int i;
unsigned int n1;
unsigned int hsdiv;
unsigned int reg137;
unsigned long int divider_max;
unsigned long int curr_div;
float curr_n1;
float n1_tmp;
float fractional;
unsigned long int fractionalI;
unsigned long int rfreqI;

/*It calculates values for current divider and maximum divider. Used for "best divider" calculation  */
divider_max=floor(FDCO_MAX/FOUT);
curr_div=ceil(FDCO_MIN/FOUT);

/* Boudaries to discriminate between large step and small step frequency change, actually is not used in
 * this implementation */

upbou=MultHi*FOUT;
lowbou=MultLo*FOUT;

ValidCombo=0;

while(curr_div <= divider_max)
/* In this loop best divider couple are calculated. It works in an iterative way. */
{
	for (i=0; i<6; i++){
		hsdiv=HSDIV[i];
		curr_n1=(float)(curr_div) / (float)(hsdiv);
		n1_tmp=floor(curr_n1);
		n1_tmp=curr_n1-n1_tmp;
		if (n1_tmp == 0.0) {
			n1=(int)curr_n1;
			if ((n1==1)||((n1&1)==0)){
				ValidCombo=1;
			}
		}
		if (ValidCombo==1) break;
	}//for
	if(ValidCombo==1) break;
	curr_div=curr_div+1;
}//while

RFREQ=(FOUT*n1*hsdiv)/FXTAL;
rfreqI=(long int)RFREQ;

fractional=RFREQ-(int)RFREQ;

fractional=fractional*POW_2_28;
fractionalI=(long int)fractional;

/* RFREQ is already well calculated according to numerical simulation 
 * Now it has to be converted to HEX in order to be programmed in si570 */

for (i=0;i<6;i++){
	REG[i]=0;
}
hsdiv=hsdiv-4;
REG[0]=(hsdiv<<5);

/* Let's try to follow application note:*/
if (n1==1)
	{ n1==0;}
else if ((n1&1)==0){
	n1=n1-1;
}

REG[0]=SetBits(REG[0],0xe0,(n1>>2));		/* write Hsdiv and N values */
REG[1]=(n1&3)<<6;							/* write N and Integer part of RFREQ values */
REG[1]=REG[1]|((rfreqI>>4)&0x3f);			/* write N and Integer part of RFREQ values */
REG[2]=(rfreqI<<4)&0xf0;					/* write Integer part of RFREQ and Fractional bit */
REG[2]=REG[2]|((fractionalI>>24)&0x0f);		/* write Integer part of RFREQ and Fractional bit */
REG[3]=((fractionalI>>16)&0xff);
REG[4]=(fractionalI>>8)&0xff;
REG[5]=fractionalI&0xff;

/* Please notice that only unsigned varibles types may be right or left shifted in a regular way 
 *  Any approach in which a signed variable type is rotated, will produce strange beahviour */

/* 137d=0x89h - read register 137 status */
reg137=read_i2c(siadd,0x89);
delay_long(5);
/* Set bit number 4 to 1 to freeze DCO */
reg137=reg137|0x10;
/* write this to si570 */
write_i2c(siadd,0x89,reg137);
delay_long(5);

/* write registers 7->12 */
for (i=0;i<6;i++){
	write_i2c(siadd,i+7,REG[i]);
}

/* read register 137 status */
reg137=read_i2c(siadd,0x89);
delay_long(5);
/* unfreeze DCO 0xef -> 1110|1111 */
reg137=reg137&0xef;
write_i2c(siadd,0x89,reg137);
delay_long(5);
/* set the "newfrequency" bit
 *  within 10ms */
write_i2c(siadd,0x87,0x40);
delay_long(3);
delay_long(3);

}//freqprog

void tx_bit1d(){
    /* this is part of the tx_debug function.*/

    DBG_DATA=1;
    delay_long(1);
    DBG_CLOCK=1;
    delay_long(1);
    DBG_CLOCK=0;
    delay_long(1);
    DBG_DATA=0;
    delay_long(1);
}

void tx_bit0d(){
    /* this is part of the tx_debug function */
    
    DBG_DATA=0;
    delay_long(1);
    DBG_CLOCK=1;
    delay_long(1);
    DBG_CLOCK=0;
    delay_long(1);
    DBG_DATA=0;
    delay_long(1);
}

void tx_bit1(){

    DFS_DATA=1;
    delay_long(6);
    DFS_CLOCK=1;
    delay_long(6);
    DFS_CLOCK=0;
    delay_long(6);
    DFS_DATA=0;
    delay_long(6);
}

void tx_bit0(){
    DFS_DATA=0;
    delay_long(6);
    DFS_CLOCK=1;
    delay_long(6);
    DFS_CLOCK=0;
    delay_long(6);
    DFS_DATA=0;
    delay_long(6);
}

void set_enable_dfs(unsigned int value){
    switch(value){
        case 0x00: //PLL
            DFS_ENABLE=0;
            delay_long(6);
            break;
        case 0x01: //PLL
            DFS_ENABLE=1;
            delay_long(6);
            break;
        case 0x02: //square 
            DFS_ENABLE=1;
            delay_long(6);
            DFS_ENABLE=0;
            delay_long(6);
    }
}

void PLL_program_dfs(unsigned int number, unsigned int *value, unsigned int direction){
    //This function will generically program any PLL
    //getting the values from *values and programming
    //only "number" bit.
    unsigned int k;

    if (direction == 0){
        k=0;
        for (k=0;k<number;k++){
            if (value[k] & 0x01){
                    tx_bit1();
            } else {
                    tx_bit0();
            }
        }
    }else {
        k=0;
        for (k=number;k>0;k--){
            if (value[k-1] & 0x01){
                    tx_bit1();
            } else {
                    tx_bit0();
            }
        }
    }
    //for

}

void ilcdfsl(unsigned int dmodel, unsigned int rfr_r, unsigned int rfr_mod, unsigned int rfn_int, unsigned int rfn_frac)
{
    unsigned int rfr[24]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned int rfn[24]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned int k,i;
/* Please notice:
 * Some registers have the same contents each time a new frequency is programmed. So, let's have them
 * statically defined. Please notice that there is a little difference between 1201 and 1301 registers
 * values.
 */    
    unsigned int master01[24]={1,1,0,1,0,0,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned int master02[24]={1,1,0,0,0,0,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned int rfctrl01[24]={0,1,0,1,0,0,1,0,0,1,1,1,0,0,0,1,0,0,0,0,0,0,0,0};
    unsigned int rfctrl02[24]={0,1,0,0,0,0,1,0,0,1,1,1,0,0,0,1,0,0,0,0,0,0,0,0};
    unsigned int ifctrl01[24]={0,1,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned int reset01[24]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    /* Let's add a model discriminator function in order to select the correct registers values*/
    
    if (dmodel){ //1301
         rfctrl01[10]=0;
         rfctrl02[10]=0;
    }    
    
    k=0;
    i=0;
    DFS_ENABLE=0;
/*Now it's time to define RF R and RF N registers, according to the user defined
 * values.
  */
    for (k=3;k<15;k++){
        if (rfr_mod & 0x01){
            rfr[k]=1;
            rfr_mod=rfr_mod>>1;
        } else {
            rfr[k]=0;
            rfr_mod=rfr_mod>>1;
        }
    }
    for (i=15;i<19;i++){
        if (rfr_r & 0x01){
            rfr[i]=1;
            rfr_r=rfr_r>>1;
        } else {
            rfr[i]=0;
            rfr_r=rfr_r>>1;
        }
    }
    //address definition
    rfr[0]=1;
    //single bit set
    rfr[19]=1;
    rfr[20]=1;
    
    /*Rf N counter array definition.     
     */
    k=0;
    i=0;
    for (k=3;k<15;k++){
        if (rfn_frac & 0x01){
            rfn[k]=1;
            rfn_frac=rfn_frac>>1;
        } else {
            rfn[k]=0;
            rfn_frac=rfn_frac>>1;
        }
    }
    for (i=15;i<23;i++){
        if (rfn_int & 0x01){
            rfn[i]=1;
            rfn_int=rfn_int>>1;
        } else {
            rfn[i]=0;
            rfn_int=rfn_int>>1;
        }
    }    
if (dmodel){ //1301
        //set_enable_dfs(2);
        PLL_program_dfs(24,reset01,1);
        delay_long(6);
        set_enable_dfs(2);
}
PLL_program_dfs(24,master01,1);
delay_long(6);
set_enable_dfs(2);

PLL_program_dfs(24,rfctrl01,1);
delay_long(6);
delay_long(6);
set_enable_dfs(2);
PLL_program_dfs(24,rfr,1);
delay_long(6);
delay_long(6);
set_enable_dfs(2);
PLL_program_dfs(24,rfn,1);
delay_long(6);
delay_long(6);
set_enable_dfs(2);
PLL_program_dfs(24,ifctrl01,1);
delay_long(6);
delay_long(6);
set_enable_dfs(2);
PLL_program_dfs(24,rfctrl02,1);
delay_long(6);
delay_long(6);
set_enable_dfs(2);
PLL_program_dfs(24,master02,1);
delay_long(6);
delay_long(6);
set_enable_dfs(2);
}

void tx_status_dfs(unsigned int model_select){
    /* This function takes care of status register programming. Different DFS
     * models are supported:
     * 1 - DFS-1301 revision C -> status 0xfc,0x5d -> 0xba3f
     * 2 - DFS-1301 revision D -> status 0xfc,0x55 -> 0xaa3f
     * 3 - Free
     * 4 - DFS-1201 revision D -> Status 0xfc,0x55 -> 0xaa3f
     * 5 - DFS-1101 revision C -> Status 0xfc,0x8d -> 0xb23f
     */
    
    unsigned int register_status;
    unsigned int k;
    unsigned int seq_bit[32];

    switch(model_select){
        case 1: register_status=0xba3f; /* DFS-1301 revision C */
                break;
        case 2: register_status=0xaa3f; /* DFS-1301 revision D */
                break;
        case 4: register_status=0xaa3f; /* DFS-1201 revision D */
                break;
        case 5: register_status=0xb23f; /* DFS-1101 revision C */
                break;
       
    }
    
    for (k=0;k<15;k++){
        if (register_status & 0x01){
            seq_bit[k]=1;
            register_status=register_status>>1;
        } else {
            seq_bit[k]=0;
            register_status=register_status>>1;
        }
    }//for
    k++;
    //set register address
    seq_bit[14]=0;
    seq_bit[15]=1;

    set_enable_dfs(0x00);
    delay_long(6);
    PLL_program_dfs(16, seq_bit,0);
    delay_long(6);
    set_enable_dfs(0x01);

}//tx_statu

void tx_na_dfs(unsigned long int value_a, unsigned int value_n_high, unsigned int value_n_low){
    //24 bit.
    unsigned long int register_na;
    unsigned int k;
    unsigned int seq_bit[32];
    for (k=0;k<32;k++){
        seq_bit[k]=0;
    }

    //register_na=(value_a<<14)|(value_n_high<<8)|(value_n_low);
    register_na=(value_a<<14)|(value_n_high<<8);
    register_na=register_na | (value_n_low);
    for (k=0;k<23;k++){
        if (register_na& 0x100000){
            seq_bit[k]=1;
            register_na=register_na<<1;
        } else {
            seq_bit[k]=0;
            register_na=register_na<<1;
        }
    }//for
    seq_bit[21]=1;
    seq_bit[22]=0;

    set_enable_dfs(0x00);
    delay_long(6);
    PLL_program_dfs(23, seq_bit,0);
    delay_long(6);
    set_enable_dfs(0x01);

}//tx_na

void tx_r_dfs(unsigned int model_select, unsigned int value){
    /* this function programs the R register value. Let's take care
     * of different DFS models:
     * 1- DFS-1301 revision c: standard value is 8 (step 5MHz) custom value is 40 (step 1MHz);
     * 1- DFS-1301 revision d: standard value is 8 (step 5MHz) custom value is 40 (step 1MHz);
     
     */
    unsigned int k;
    unsigned int seq_bit[32];
    for (k=0;k<32;k++){
        seq_bit[k]=0;
    }
    switch(model_select){
        case 1:                               //DFS-1301                    
        case 2: switch(value){
                   case 1: seq_bit[12]=1;     /* V alue is 8 */
                           break;
                   case 2: seq_bit[10]=1;
                           seq_bit[12]=1;     /* Value is 40 */
                           break;
                }
                break;
        case 4: switch(value){                //DFS-1201
                   case 1: seq_bit[12]=1;     /* V alue is 8 */
                           break;
                   case 2: seq_bit[10]=1;
                           seq_bit[12]=1;     /* Value is 40 */
                           break;
                    case 3:seq_bit[12]=1;     /* Value is 12 */
                           seq_bit[13]=1;
                           break;
                }
                break;
        case 5: switch(value){
                    case 1: seq_bit[10]=1;   /* Value is 37 */
                            seq_bit[13]=1;
                            seq_bit[15]=1;
                            break;
                    case 2: seq_bit[8]=1;    /* Value is 185 */
                            seq_bit[10]=1;
                            seq_bit[11]=1;
                            seq_bit[12]=1;
                            seq_bit[15]=1;
        }
                break;
    }
    
    //address programming
    seq_bit[16]=1;
    seq_bit[17]=1;

    set_enable_dfs(0x00);
    delay_long(6);
    PLL_program_dfs(18, seq_bit,0);
    delay_long(6);
    set_enable_dfs(0x01);
}//tx_r

void initial_setup_dfs(unsigned int model_select){
    switch(model_select){
        case 1:/* DFS-1301 Revision C */
            tx_status_dfs(1);
            break;
        case 2:/* DFS-1301 Revision D */
            tx_status_dfs(2);
            break;
        case 4:/* DFS-1201 Revision D */
            tx_status_dfs(4);
            break;
        case 5: /* DFS-1101 Revision C */
            tx_status_dfs(5);
            break;
    }
    delay_long(6);
}//initial_setup

void initial_values_dfs(unsigned int model_select){
    switch(model_select){
        case 1: /* DFS-1301 Revision C*/
                /* Program 13000 MHz Output */
                tx_r_dfs(1,1);
                delay_long(6);
                tx_na_dfs(0x08,0x00,0x1b);
                delay_long(6);
                break;
        case 2: /* DFS_1301 Revision D*/
                tx_r_dfs(2,1);
                delay_long(6);
                tx_na_dfs(0x00,0x00,0x37);
                delay_long(6);
                break;
        case 4: /* DFS_1201 Revision D*/
                tx_r_dfs(4,3);
                delay_long(6);
                tx_na_dfs(0x00,0x00,0x2d);
                delay_long(6);
                break;
        case 5: /* DFS_1101 Revision C*/
                tx_r_dfs(5,1);
                delay_long(6);
                tx_na_dfs(0x02,0x00,0x19);
                delay_long(6);
                break;
    }
}//initial_values

void program(unsigned int model_select, unsigned int value){
    /* Let's use come kind of standard for value:
     * 1 - beacon frequency
     * 2 - assume 144MHz  IF
     * 3 - assume 432MHz  IF
     * 4 - assume 1296MHz IF
     */
     
    switch(model_select){
        case 1: /* DFS-1301 Revision C
                 * Assume R=40 in order to have 1 MHz minimum step.
                 * FINAL       IF       LO      LO-DFS  R   N   Nhex    A   Ahex
                 * 72032        Beacon  --      12672   40  112  70     80  50
                 * 76032        144     75888   12648   40  110  6E     88  58
                 * 76032        432     75600   12600   8   22   16      8   8
                 * 76032        1286    74736   12456   40  98   62     88  58
                 * Test Fields -------------------------------------------------
                 * 12650                                8   23   17     2   2
                 * 13000                                8   27   1B     8   8
                 * 13200                                8   29   1D     16  10
                 * 13000                                40  132  84     88  58
                 */
               switch(value){
                   case 0x01: //program 12672; Beacon -> R=40
                        tx_status_dfs(1);
                        delay_long(2);
                        tx_r_dfs(1,2);  
                        delay_long(2);
                        tx_na_dfs(0x50,0x00,0x70); //a,n
                        delay_long(2);
                    break;
                   case 0x02:  //program 12648; IF=144MHz -> R=40
                        tx_status_dfs(1);
                        delay_long(2);
                        tx_r_dfs(1,2);
                        delay_long(2);
                        tx_na_dfs(0x58,0x00,0x6e); //a, n
                        delay_long(2);
                    break;
                   case 0x03:  //program 12600; IF=432MHz R=8
                        tx_status_dfs(1);
                        delay_long(2);
                        tx_r_dfs(1,1);
                        delay_long(2);
                        tx_na_dfs(0x08,0x00,0x16); //a, n
                        delay_long(2);
                    break;
                   case 0x04:  //program 12456; IF=1296MHz 
                        tx_status_dfs(1);
                        delay_long(2);
                        tx_r_dfs(1,2);
                        delay_long(2);
                        tx_na_dfs(0x58,0x00,0x62); //a, n
                        delay_long(2);
                    break;
                    case 0x05:  //program 12650; TEST R=8
                        tx_status_dfs(1);
                        delay_long(2);
                        tx_r_dfs(1,1);
                        delay_long(2);
                        tx_na_dfs(0x02,0x00,0x17); //a, n
                        delay_long(2);
                    break;
                    case 0x06:  //program 13200; TEST R=8
                        tx_status_dfs(1);
                        delay_long(2);
                        tx_r_dfs(1,1);
                        delay_long(2);
                        tx_na_dfs(0x10,0x00,0x1D); //a, n
                        delay_long(2);
                    break;
                    case 0x07:  //program 13000; TEST R=8
                        tx_status_dfs(1);
                        delay_long(2);
                        tx_r_dfs(1,1);
                        delay_long(2);
                        tx_na_dfs(0x08,0x00,0x1B); //a, n
                        delay_long(2);
                    break;
                    case 0x08:  //program 13000; TEST R=40
                        tx_status_dfs(1);
                        delay_long(2);
                        tx_r_dfs(1,2);
                        delay_long(2);
                        tx_na_dfs(0x58,0x00,0x84); //a, n
                        delay_long(2);
                    break;
               }
               break;  
        case 2: /*DFS-1301 Revision D
                 * 
                 * * Assume R=40 in order to have 1 MHz minimum step.
                 * FINAL       IF       LO      LO-DFS  R   N   Nhex    A   Ahex
                 * 72032        Beacon  --      12672   40  234 EA      0   0
                 * 76032        144     75888   12648   40  230 E6      8   8
                 * 76032        432     75600   12600   8   45  2D      0   0
                 * 76032        1286    74736   12456   40  206 CE      8   8
                 * Test Fields -------------------------------------------------
                 * 12650                                8   46  2E      2   2                       
                 * 13000                                8   55  37      0   0
                 * 13200                                8   60  3C      0   0
                 * 13000                                40  274 112     8   8
                 *
                 */
            switch(value){
                case 0x01: //program 12672;
                    tx_status_dfs(2);
                    delay_long(2);
                    tx_r_dfs(2,2);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0xea); //a,n
                    delay_long(2);
                    break;
                case 0x02:  //program 12648; IF=144
                    tx_status_dfs(2);
                    delay_long(2);
                    tx_r_dfs(2,2);
                    delay_long(2);
                    tx_na_dfs(0x08,0x00,0xe6); //a, n
                    delay_long(2);
                    break;
                case 0x03:  //program 12600; IF=432
                    tx_status_dfs(2);
                    delay_long(2);
                    tx_r_dfs(2,1);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0x2d); //a, n
                    delay_long(2);
                    break;
                case 0x04:  //program 12456; IF=1296
                    tx_status_dfs(2);
                    delay_long(2);
                    tx_r_dfs(2,2);
                    delay_long(2);
                    tx_na_dfs(0x08,0x00,0xce); //a, n
                    delay_long(2);
                    break;
                case 0x05:  //program 12650; TEST R=8
                    tx_status_dfs(2);
                    delay_long(2);
                    tx_r_dfs(2,1);
                    delay_long(2);
                    tx_na_dfs(0x02,0x00,0x2e); //a, n
                    delay_long(2);
                    break;
                case 0x06:  //program 13200; TEST R=8
                    tx_status_dfs(2);
                    delay_long(2);
                    tx_r_dfs(2,1);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0x3c); //a, n
                    delay_long(2);
                    break;
                case 0x07:  //program 13000; TEST R=8
                    tx_status_dfs(2);
                    delay_long(2);
                    tx_r_dfs(2,1);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0x37); //a, n
                    delay_long(2);
                    break;
                case 0x08:  //program 13000; TEST R=40
                    tx_status_dfs(2);
                    delay_long(2);
                    tx_r_dfs(2,2);
                    delay_long(2);
                    tx_na_dfs(0x08,0x01,0x12); //a, n
                    delay_long(2);
                    break;
               }
            break;
        case 4: /*DFS-1201 Revision D
                 * 
                 * * Assume R=40 in order to have 1 MHz minimum step.
                 * FINAL       IF       LO      LO-DFS  R   N   Nhex    A   Ahex
                 * 47088        Beacon  --      11772   40  121 79      4   4
                 * 47088        144     46944   11736   40  116 74      8   8
                 * 47088        432     45656   11414   40  76  4c      6   6
                 * 47088        1286    45792   11448   40  80  50      8   8
                 * ------------------------------------------------------------
                 * FINAL       IF       LO      LO-DFS  R   N   Nhex    A   Ahex
                 * 24048        beacon  --      12024   40  154 99      0   0                 
                 * 24048        144     23904   11952   40  143 8F      8   8                  
                 * 24048        432     23616   11808   40  126 7E      0   0                 
                 * 24048        1296    22752   11376   40  72  48      0   0    
                 * -----------------------------------------------------------
                 * FINAL        R   N   Nhex    A   Ahex
                 * 11200        
                 * 12000
                 *
                 */
                switch(value){
                  case 0x01: //program 11772;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,2);
                    delay_long(2);
                    tx_na_dfs(0x04,0x00,0x79); //a,n
                    delay_long(2);
                    break;
                  case 0x02: //program 11736;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,2);
                    delay_long(2);
                    tx_na_dfs(0x08,0x00,0x74); //a,n
                    delay_long(2);
                    break;
                  case 0x03: //program 11414;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,2);
                    delay_long(2);
                    tx_na_dfs(0x06,0x00,0x4c); //a,n
                    delay_long(2);
                    break;
                  case 0x04: //program 11448;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,2);
                    delay_long(2);
                    tx_na_dfs(0x08,0x00,0x50); //a,n
                    delay_long(2);
                    break;
                  case 0x05: //program 12024;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,2);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0x99); //a,n
                    delay_long(2);
                    break;
                  case 0x06: //program 11952;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,2);
                    delay_long(2);
                    tx_na_dfs(0x08,0x00,0x8f); //a,n
                    delay_long(2);
                    break;
                  case 0x07: //program 11808;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,2);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0x7e); //a,n
                    delay_long(2);
                    break;
                  case 0x08: //program 11376;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,2);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0x48); //a,n
                    delay_long(2);
                    break;
                  case 0x09: //program 11200;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,1);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0x0a); //a,n
                    delay_long(2);
                    break;
                  case 0x0A: //program 12000;
                    tx_status_dfs(4);
                    delay_long(2);
                    tx_r_dfs(4,1);
                    delay_long(2);
                    tx_na_dfs(0x00,0x00,0x1e); //a,n
                    delay_long(2);
                    break;
                }//value
                break;
        case 5: /*DFS-1101 Revision C
                 * 
                 * * Assume R=185 in order to achieve 1MHz step
                 * FINAL        IF       LO-DFS  R   N      Nhex    A   Ahex
                 * 10368        144      10512   185 65     0x41    2   0x02
                 * 10368        432      10800   185 100    0x64    10  0x0A
                 * 10575        ---      -----   37  13     0x0D    13  0x0D
                 * 11400        ---      -----   37  35     0x23    2   0x02
                 * 11000        ---      -----   37  25     0x19    2   0x02
                 * 10500        ---      -----   37  36     0x24    14  0x0E
                 */
                switch(value){
                case 0x01: //program 10512;
                    tx_status_dfs(5);
                    delay_long(2);
                    tx_r_dfs(5,2);
                    delay_long(2);
                    tx_na_dfs(0x02,0x00,0x41); //a,n
                    delay_long(2);
                    break;
                case 0x02: //program 10800;
                    tx_status_dfs(5);
                    delay_long(2);
                    tx_r_dfs(5,2);
                    delay_long(2);
                    tx_na_dfs(0x0A,0x00,0x64); //a,n
                    delay_long(2);
                    break;
                case 0x03: //program 10575;
                    tx_status_dfs(5);
                    delay_long(2);
                    tx_r_dfs(5,1);
                    delay_long(2);
                    tx_na_dfs(0x0D,0x00,0x0D); //a,n
                    delay_long(2);
                    break;
                case 0x04: //program 11400;
                    tx_status_dfs(5);
                    delay_long(2);
                    tx_r_dfs(5,1);
                    delay_long(2);
                    tx_na_dfs(0x02,0x00,0x23); //a,n
                    delay_long(2);
                    break;
                case 0x05: //program 11000;
                    tx_status_dfs(5);
                    delay_long(2);
                    tx_r_dfs(5,1);
                    delay_long(2);
                    tx_na_dfs(0x02,0x00,0x19); //a,n
                    delay_long(2);
                    break;
                case 0x06: //program 10500;
                    tx_status_dfs(5);
                    delay_long(2);
                    tx_r_dfs(5,1);
                    delay_long(2);
                    tx_na_dfs(0x0E,0x00,0x24); //a,n
                    delay_long(2);
                    break;
                }
                break;
        case 0x0A:/*ILCDFSL-1201
                 * Assume R=40 in order to have 1 MHz minimum step.
                 * FINAL       IF       LO      LO-DFS  MOD     MODhex  R   Frac    FracH   INT INTh
                 * 47088        Beacon  --      11772   50      32      1   36      24      117 75
                 * 47088        144     46944   11736   50      32      1   18      12      117 75
                 * 47088        432     45656   11414   50      32      1   7       7       114 72
                 * 47088        1286    45792   11448   50      32      1   24      18      114 72
                 * ------------------------------------------------------------
                 * FINAL       IF       LO      LO-DFS  MOD     MODhex  R   Frac    FracH   INT INTh
                 * 24048        beacon  --      12024   50      32      1   12      C       120 78
                 * 24048        144     23904   11952   50      32      1   26      1A      119 77
                 * 24048        432     23616   11808   50      32      1   4       4       118 76
                 * 24048        1296    22752   11376   50      32      1   38      26      113 71
                 * ---------------------------------------------------------------------------------
                 * FINAL        USE     MOD     MODhex  R   Frac    FracH   INT     INTh
                 * 11200        test     10     A       1   10      A       111     6F
                 * 11600        test     10     A       1   10      A       115     73     
                 * 12000        test     10     A       1   10      A       119     77
                 * 12000        test     50     32      1   50      32      119     77                 
                 */
            switch(value){ //
                case 0x01: //program 11772 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x75,0x24);
                        delay_long(2);
                        break;
                case 0x02: //program 11736 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x75,0x12);
                        delay_long(2);
                        break;
                case 0x03: //program 11414 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x72,0x07);
                        delay_long(2);
                        break;
                case 0x04: //program 11448 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x72,0x18);
                        delay_long(2);
                        break;
                case 0x05: //program 12024 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x78,0x0C);
                        delay_long(2);
                        break;
                case 0x06: //program 11952 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x77,0x1A);
                        delay_long(2);
                        break;
                case 0x07: //program 11808 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x76,0x04);
                        delay_long(2);
                        break;
                case 0x08: //program 11376 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x71,0x26);
                        delay_long(2);
                        break;
                case 0x09: //program 11200 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x0A,0x6f,0x0a);
                        delay_long(2);
                        break;        
                case 0x0A: //program 12000 mOD=10 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x0a,0x77,0x0a);
                        delay_long(2);
                        break;
                case 0x0B: //program 12000 mod=50(ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x00,0x01,0x32,0x77,0x32);
                        delay_long(2);
                        break;
            }
            break;
            case 0x0B:/*ILCDFSL-1301 Eraldo
                 * 
                 * FINAL    R   D   Mod ModH    Frac    FracH   Int IntH
                 * 10318    1   1   40  28      39      27      128 89
                 * 9984     1   1   40  28      128     80      39  27
                 * 10368,5  1   1   160 A0      97      61      129 81  
                 * 10368,4  1   1   200 C8      121     79      129 81
                 * 10368,2  1   1   400 190     241     F1      129 81
                 * 10224    1   1   5   5       4       4       127 7F
                 * 10368,3  1   1   800 320     483     1E3     129 81
                 */
            switch(value){ //
                case 0x01: //program 10318 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x28,0x89,0x27);
                        delay_long(2);
                        break;
                case 0x02: //program 9984 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x28,0x27,0x80);
                        delay_long(2);
                        break;
                case 0x03: //program 10368,5 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0xA0,0x81,0x61);
                        delay_long(2);
                        break;
                case 0x04: //program 10368,4 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0xC8,0x81,0x79);
                        delay_long(2);
                        break;
                case 0x05: //program 10368,3 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x320,0x81,0x1e3);
                        delay_long(2);
                        break;
                case 0x06: //program 10368,2 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x190,0x81,0xF1);
                        delay_long(2);
                        break;
                case 0x07: //program 10224 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x5,0x7F,0x4);
                        delay_long(2);
                        break;
            }
            break;
            //case 0x0C:
                /*ILCDFSL-1301 Generic
                 *
                 * FINAL    R   D   Mod ModH    Frac    FracH   Int IntH
                 * 12672    1   1   5   5       7       7       157 9D
                 * 12648    1   1   10  A       11      B       157 9D
                 * 12600    1   1   2   2       17      11      149 95
                 * 12456    1   1   10  A       7       7       155 9B
                 * 12650    1   1   40  28      5       5       158 9E
                 * 13000    1   1   2   2       15      F       155 9B
                 * 13000    1   1   20  14      10      A       162 A2
                 * 13200    1   1   1   1       8       8       157 9D
                 */
            /*switch(value){ //
                case 0x01: //program 12672 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x05,0x9D,0x07);
                        delay_long(2);
                        break;
                case 0x02: //program 12648 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x0A,0x9D,0x0B);
                        delay_long(2);
                        break;
                case 0x03: //program 12600 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x02,0x95,0x11);
                        delay_long(2);
                        break;
                case 0x04: //program 12456 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x0A,0x9B,0x07);
                        delay_long(2);
                        break;
                case 0x05: //program 12650 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x28,0x9E,0x05);
                        delay_long(2);
                        break;
                case 0x06: //program 13000 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x02,0x9B,0x0F);
                        delay_long(2);
                        break;
                case 0x07: //program 13000 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x14,0xA2,0x0A);
                        delay_long(2);
                        break;
                case 0x08: //program 13200 (ilcdfsl (R,Mod,Int,Frac));
                        ilcdfsl(0x01,0x01,0x01,0x9D,0x08);
                        delay_long(2);
                        break;
            }
             break;*/
    }//switch

}

void tx_debug(unsigned int *value){
    /* This function performs some kind of debug. Is the only way to have 
     register in a graphical  way using an oscilloscope. Register are passed
     via pointer and then are transmitted via a serial synchronous interface.*/
    unsigned int k,m;
    
   DBG_DATA=0;
   DBG_CLOCK=0;
   
   for (k=0;k<6;k++)
       for (m=0;m<8;m++){
           if (value[k] & (0x80u >> m)){
               tx_bit1d(); }
           else {
               tx_bit0d();
           }
       }
    delay_long(2);
}

void read_startup(){
/* This function performs reset and reading of si570 registers. 
 * Registers are then transmitted via the serial bit bang interface
 * so that their value can be verified. More over the Fxtal value
 * is calculated in order to make frequency programming more
 * accurate. This operation is very memory intensive! 
 */
unsigned int i,k;
/* Reset device and load default values */ 

write_i2c(0x55,0x87,0x01);
delay_long(2);

REG[0]=read_i2c(0x55,0x07);
REG[1]=read_i2c(0x55,0x08);
REG[2]=read_i2c(0x55,0x09);
REG[3]=read_i2c(0x55,0x0a);
REG[4]=read_i2c(0x55,0x0b);
REG[5]=read_i2c(0x55,0x0c);

/* Call the bit bang debug interface */
tx_debug(REG);

INITIAL_HSDIV=((REG[0]&0xe0)>>5)+4;                     //Tiene conto della codifica di HSDIV - valore reale
INITIAL_N1=((REG[0]&0x1f)<<2)+((REG[1]&0xc0)>>6);	
if (INITIAL_N1 == 0){
	INITIAL_N1=1;
} else if ((INITIAL_N1&1) != 0) {
	INITIAL_N1=INITIAL_N1+1;
}														//TIene conto del fatto che N1 pu assumere solo valori pari o 1;

//Double Floating Point
FRAC_BITS=(REG[2]&0x0f)*POW_2_24;
FRAC_BITS=FRAC_BITS+(REG[3]*POW_2_16);
FRAC_BITS=FRAC_BITS+(REG[4]*256);
FRAC_BITS=FRAC_BITS+REG[5];

RFREQ=FRAC_BITS;
RFREQ=RFREQ/POW_2_28;

/* Please notice: RFREQ is 38 bit long. Typical values for frequency are 47MHz to 56 MHz. Those values can
 * fit in a 34 bit long register. 34 is longer than 32, so last two bit are discarded so that RFREQ can fit in
 * a long int.
 */

INITIAL_RFREQ_LONG_INT=(REG[1]&0x3f);
INITIAL_RFREQ_LONG_INT=(INITIAL_RFREQ_LONG_INT<<8)+(REG[2]&0xF0);
INITIAL_RFREQ_LONG_INT=(INITIAL_RFREQ_LONG_INT>>4);

INITIAL_RFREQ_LONG_FRAC=(REG[2]&0x0F);
INITIAL_RFREQ_LONG_FRAC=(INITIAL_RFREQ_LONG_FRAC<<8)+(REG[3]);
INITIAL_RFREQ_LONG_FRAC=(INITIAL_RFREQ_LONG_FRAC<<8)+(REG[4]);
INITIAL_RFREQ_LONG_FRAC=(INITIAL_RFREQ_LONG_FRAC<<8)+(REG[5]);
RFREQ=RFREQ+(((REG[1]&0x3f)<<4)+((REG[2]&0xf0)>>4));

/* This is the most important part! */
FXTAL=(FOUT0_START_UP*INITIAL_N1*INITIAL_HSDIV)/RFREQ;

/* These values are very important when using Si570 as a VFO*/
upbou=MultHi*FOUT0_START_UP;
lowbou=MultLo*FOUT0_START_UP;

delay_long(3);
delay_long(3);

}//read_startup

void blink_led(unsigned int nblink, unsigned int speed){
    /* Just a blink led 1 routine.
     * nblink: number of blinks
     * speed : n*200ms delay
     * speed : if =1 then 500ms delay     
     */
    
    unsigned int i;
    LED1=0;
    if (nblink==0){
        LED1=1;
        for (i=0;i<=speed;i++){
            delay_long(3); //200ms
        }
    }
    for (i=0;i<nblink;i++){
        LED1=1;
        if (speed==1){
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
        } else {
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
        }

        LED1=0;
        if (speed==1){
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
        } else {
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
        }
    }
    LED1=0;
}

blink_led2(unsigned int nblink, unsigned int speed){
    unsigned int i;
    LED2=0;
    if (nblink==0){
        LED2=1;
        for (i=0;i<=speed;i++){
            delay_long(3); //200ms
        }
    }
    for (i=0;i<nblink;i++){
        LED2=1;
        if (speed==1){
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
        } else {
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
        }

        LED2=0;
        if (speed==1){
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
        } else {
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
            delay_long(3); //200ms
        }
    }
    LED2=0;
}


void wait_5s(void){
    delay_long(4);
    delay_long(4);
    delay_long(4);
    delay_long(4);
    delay_long(4);
}

void led_numbers(unsigned int value){
    LED1=0;
    LED2=0;    
    switch(value){
        case 0:
            LED2=0;
            LED1=0;
            break;        
        case 1:
            LED2=1;
            LED1=0;
            break;
        case 2:
            LED1=1;
            LED2=0;
            break;
        case 3:
            LED1=1;
            LED2=1;
            break;
    }
    
}

void test_dfs(unsigned int model_value){
    LED1=1;
    LED2=1;
    delay_long(4);
    LED1=0;
    LED2=0;
    delay_long(4);
    LED1=1;
    LED2=1;
    delay_long(4);
    LED1=0;
    LED2=0;
    delay_long(4);
    
    switch(model_value){
        case 0x01:              //tested is OK DFS-1301 C
            led_numbers(0);
            program(1,5);       //12650 R8
            wait_5s();
            led_numbers(1);
            program(1,6);       //13200 R8
            wait_5s();
            led_numbers(2);
            program(1,7);       //13000 R8
            wait_5s();
            led_numbers(3);
            program(1,8);       //13000 R40
            wait_5s();
            led_numbers(0);
            program(1,1);       //12672 R40
            wait_5s();
            led_numbers(1);
            program(1,2);       //12648 R40
            wait_5s();
            led_numbers(2);
            program(1,3);       //12600 R8
            wait_5s();
            led_numbers(3);
            program(1,4);       //12456 R40
            wait_5s();
        break;
        case 0x02:              // DFS-1301 D
            led_numbers(0);
            program(2,5);       //12650 R8
            wait_5s();
            led_numbers(1);
            program(2,6);       //13200 R8
            wait_5s();
            led_numbers(2);
            program(2,7);       //13000 R8
            wait_5s();
            led_numbers(3);
            program(2,8);       //13000 R40
            wait_5s();
            led_numbers(0);
            program(2,1);       //12672 R40
            wait_5s();
            led_numbers(1);
            program(2,2);       //12648 R40
            wait_5s();
            led_numbers(2);
            program(2,3);       //12600 R8
            wait_5s();
            led_numbers(3);
            program(2,4);       //12456 R40
            wait_5s();
            break;
        case 0x04:              //DFS-1201 D
            led_numbers(0);
            program(4,5);       //12024 R40
            wait_5s();
            led_numbers(1);
            program(4,6);       //11952 R40
            wait_5s();
            led_numbers(2);
            program(4,7);       //11808 R40
            wait_5s();
            led_numbers(3);
            program(4,8);       //11376 R40
            wait_5s();
            led_numbers(0);
            program(4,1);       //11772 R40
            wait_5s();
            led_numbers(1);
            program(4,2);       //11736 R40
            wait_5s();
            led_numbers(2);
            program(4,9);       //11200 R8
            wait_5s();
            led_numbers(3);
            program(4,0x0a);    //12000 R8
            wait_5s();
            led_numbers(0);
            program(4,3);       //11414 R40
            wait_5s();
            led_numbers(1);
            program(4,4);       //11448 R40
            wait_5s();
            break;
        case 0x05:
            led_numbers(0);     //DFS-1101
            program(5,1);       //10224 R185
            wait_5s();
            led_numbers(1);
            program(5,2);       //10512 R185
            wait_5s();
            led_numbers(2);
            program(5,3);       //10800 R185
            wait_5s();
            led_numbers(3);
            program(5,4);       //11664 R185
            wait_5s();
            led_numbers(0);
            program(5,5);       //10575 R37
            wait_5s();
            led_numbers(1);
            program(5,6);       //11400 R37
            wait_5s();
            led_numbers(2);
            program(5,7);       //11000 R37
            wait_5s();
            break;
        case 0x0A:
            led_numbers(0);     //ILCDFSL-1201
            program(0x0A,0x01);    //11772
            wait_5s();
            led_numbers(1);     
            program(0x0A,0x02);    //11736
            wait_5s();
            led_numbers(2);     
            program(0x0A,0x03);    //11414
            wait_5s();
            led_numbers(0);     
            program(0x0A,0x04);    //11448
            wait_5s();
            led_numbers(1);    
            program(0x0A,0x05);    //12024
            wait_5s();
            led_numbers(2);     
            program(0x0A,0x06);    //11952
            wait_5s();
            led_numbers(0);     
            program(0x0A,0x07);    //11808
            wait_5s();
            led_numbers(1);    
            program(0x0A,0x08);    //11376
            wait_5s();
            led_numbers(2);     
            program(0x0A,0x09);    //11200
            wait_5s();
            led_numbers(0);     
            program(0x0A,0x0a);    //12000
            wait_5s();
            led_numbers(1);     
            program(0x0A,0x0b);    //12000
            wait_5s();
            break;
        case 0x0B:
            led_numbers(0);         //ILCDFSL-1301 Eraldo
            program(0x0B,0x01);     //10318
            wait_5s();
            led_numbers(1);         
            program(0x0B,0x02);     //9984
            wait_5s();
            led_numbers(2);         
            program(0x0B,0x03);     //10368,5
            wait_5s();
            led_numbers(0);         
            program(0x0B,0x04);     //10368,4
            wait_5s();
            led_numbers(1);         
            program(0x0B,0x05);     //10368,3
            wait_5s();
            led_numbers(2);         
            program(0x0B,0x06);     //10368,2
            wait_5s();
            led_numbers(0);         
            program(0x0B,0x07);     //10224
            wait_5s();
            break;
        case 0x0C:
            led_numbers(0);         //ILCDFSL-1301
            program(0x0C,0x01);     //12672
            wait_5s();
            led_numbers(1);         
            program(0x0C,0x02);     //12648
            wait_5s();
            led_numbers(2);         
            program(0x0C,0x03);     //12600
            wait_5s();
            led_numbers(0);         
            program(0x0C,0x04);     //12456
            wait_5s();
            led_numbers(1);         
            program(0x0C,0x05);     //12650
            wait_5s();
            led_numbers(2);         
            program(0x0C,0x06);     //13000
            wait_5s();
            led_numbers(0);         
            program(0x0C,0x07);     //13000
            wait_5s();
            led_numbers(1);         
            program(0x0C,0x07);     //13200
            wait_5s();
            break;
    }
    
}


void main() {
    
    unsigned int frequency_cfg;
    unsigned int hardware_cfg;
    
    //LED1=0;
    //LED2=0;
    DFS_DATA=0;
    DFS_CLOCK=0;
    DFS_ENABLE=0;
        
    initHardware();
    LED1=0;
    LED2=0;
    
    delay_long(4);
    delay_long(4);
    delay_long(4);
    delay_long(4);
    LED2=1;
    
    set_enable_dfs(0x01);
    delay_long(4);
    delay_long(4);
    delay_long(4);
    delay_long(4);
    LED2=0;
    
    
    if (DSW1) {
        //Si570 enable switch
        read_startup();
        FreqProg(100.000000);
        blink_led(3,1);
    }
    delay_long(4);
    
    //Frequency configuration 
    hardware_cfg=((PORTA & 0x1E) >>1) & 0x0F;
    frequency_cfg=((PORTC & 0x80)>>7); //rc7
    frequency_cfg=(frequency_cfg | ( (PORTC & 0x40)>> 5) );
    frequency_cfg=(frequency_cfg | (PORTC & 0x04));
    frequency_cfg=(frequency_cfg | ((PORTA & 0x20)>>2 ));
    frequency_cfg=frequency_cfg & 0x0F;
            
    switch (hardware_cfg){
        case 0x08:  // Dip Switch Configuration: X 0001 XXXX
                    //DFS-1301 Rev. C
            blink_led(1,1);
            switch (frequency_cfg) {
                case 0x01 : blink_led2(1,1);
                            program(0x01,0x01); //12672
                            break;
                case 0x02 : blink_led2(2,1);
                            program(0x01,0x02); //12648
                            break;
                case 0x03 : blink_led2(3,1);
                            program(0x01,0x03); //12600
                            break;
                case 0x04 : blink_led2(4,1);
                            program(0x01,0x04); //12456
                            break;
                case 0x05 : blink_led2(5,1);
                            program(0x01,0x05); //12650
                            break;
                case 0x06 : blink_led2(6,1);
                            program(0x01,0x06); //13200
                            break;
                case 0x07 : blink_led2(7,1);
                            program(0x01,0x07); //13000
                            break;
                case 0x08 : blink_led2(8,1);
                            program(0x01,0x08); //13000
                            break;            
                default:    blink_led2(10,1);
                            break;  
            }//switch software
            break;
        case 0x04:  // Dip Switch Configuration: X 0010 XXXX
                    //DFS-1301 Rev. D
            blink_led(2,1);
            switch (frequency_cfg) {
                case 0x01 : blink_led2(1,1);
                            program(0x02,0x01); //12672
                            break;
                case 0x02 : blink_led2(2,1);
                            program(0x02,0x02); //12648
                            break;
                case 0x03 : blink_led2(3,1);
                            program(0x02,0x03); //12600
                            break;
                case 0x04 : blink_led2(4,1);
                            program(0x02,0x04); //12456
                            break;
                case 0x05 : blink_led2(5,1);
                            program(0x02,0x05); //12650
                            break;
                case 0x06 : blink_led2(6,1);
                            program(0x02,0x06); //13200
                            break;
                case 0x07 : blink_led2(7,1);
                            program(0x02,0x07); //13000
                            break;
                case 0x08 : blink_led2(8,1);
                            program(0x02,0x08); //13000
                            break;
                default:    blink_led2(10,1);
                            break;
            }//switch software
            break;
        case 0x0C:  // Dip Switch Configuration: X 0011 XXXX
                    //UNUSED
            blink_led(3,1);
            break;
        case 0x02:  // Dip Switch Configuration: X 0100 XXXX
                    //DFS-1201 Rev. D
            blink_led(4,1);
            switch (frequency_cfg) {
                case 0x01 : blink_led2(1,1);
                            program(0x04,0x01); //11772
                            break;
                case 0x02 : blink_led2(2,1);
                            program(0x04,0x02); //11736
                            break;
                case 0x03 : blink_led2(3,1);
                            program(0x04,0x03); //11414
                            break;
                case 0x04 : blink_led2(4,1);
                            program(0x04,0x04); //11448
                            break;
                case 0x05 : blink_led2(5,1);
                            program(0x04,0x05); //12024
                            break;
                case 0x06 : blink_led2(6,1);
                            program(0x04,0x06); //11952
                            break;
                case 0x07 : blink_led2(7,1);
                            program(0x04,0x07); //11808
                            break;
                case 0x08 : blink_led2(8,1);
                            program(0x04,0x08); //11376
                            break;            
                case 0x09 : blink_led2(9,1);
                            program(0x04,0x09); //11200
                            break;            
                case 0x0A : blink_led2(0x0A,1);
                            program(0x04,0x0A); //12000
                            break;            
                default:    blink_led2(10,1);
                            break;
            }//switch software
            break;
        case 0x0A:  // Dip Switch Configuration: X 0101 XXXX
                    //DFS-1101 Rev. C
            blink_led(5,1);
            switch (frequency_cfg) {
                case 0x01 : blink_led2(1,1);
                            program(0x05,0x01); //10512
                            break;   
                case 0x02 : blink_led2(2,1);
                            program(0x05,0x02); //10800
                            break;     
                case 0x03 : program(0x05,0x03); //10575
                            break;     
                case 0x04 : program(0x05,0x04); //11400
                            break;     
                case 0x05 : program(0x05,0x05); //11000
                            break;     
                case 0x06 : program(0x05,0x06); //10500
                            break;
                default:    blink_led2(10,1);
                            break;            
            }
            break;
        case 0x06:  // Dip Switch Configuration: X 0101 XXXX
                    //ILCDFSL-1201 
            delay_long(4);
            delay_long(4);
            blink_led(6,1);
            switch (frequency_cfg) {
                case 0x01 : blink_led2(1,1);
                            program(0x0A,0x01); //11772
                            break; 
                case 0x02 : blink_led2(2,1);
                            program(0x0A,0x02); //11736
                            break;
                case 0x03 : blink_led2(3,1);
                            program(0x0A,0x03); //11414
                            break;
                case 0x04 : blink_led2(4,1);
                            program(0x0A,0x04); //11448
                            break;
                case 0x05 : blink_led2(5,1);
                            program(0x0A,0x05); //12024
                            break;
                case 0x06 : blink_led2(6,1);
                            program(0x0A,0x06); //11952
                            break;
                case 0x07 : blink_led2(7,1);
                            program(0x0A,0x07); //11808
                            break;
                case 0x08 : blink_led2(8,1);
                            program(0x0A,0x08); //11376
                            break;
                case 0x09 : blink_led2(9,1);
                            program(0x0A,0x09); //11200
                            break;
                case 0x0A : blink_led2(0x0A,1);
                            program(0x0A,0x0A); //12000
                            break;
                case 0x0B : blink_led2(0x0B,1);
                            program(0x0A,0x0B); //12000
                            break;
                default:    blink_led2(10,1);
                            break;            
            }
            break;
        case 0x0E:  // Dip Switch Configuration: X 0111 XXXX
                    //ILCDFSL-1301 I4SBX
            blink_led(7,1);
            switch (frequency_cfg) {
                case 0x01 : blink_led2(1,1);
                            program(0x0B,0x01); //10318
                            break; 
                case 0x02 : blink_led2(2,1);
                            program(0x0B,0x02); //9984
                            break; 
                case 0x03 : blink_led2(3,1);
                            program(0x0B,0x03); //10368,5
                            break; 
                case 0x04 : blink_led2(4,1);
                            program(0x0B,0x04); //10368,4
                            break; 
                case 0x05 : blink_led2(5,1);
                            program(0x0B,0x05); //10368,3
                            break; 
                case 0x06 : blink_led2(6,1);
                            program(0x0B,0x06); //10368,2
                            break; 
                case 0x07 : blink_led2(7,1);
                            program(0x0B,0x07); //10224
                            break;
                default:    blink_led2(10,1);
                            break;            
            }
            break;
        case 0x01:  // Dip Switch Configuration: X 1000 XXXX
                    //ILCDFSL-1301 
            blink_led(8,1);
            switch (frequency_cfg) {
                case 0x01 : blink_led2(1,1);
                            program(0x0C,0x01); //12672
                            break;
                case 0x02 : blink_led2(2,1);
                            program(0x0C,0x01); //12648
                            break;
                case 0x03 : blink_led2(3,1);
                            program(0x0C,0x01); //12600
                            break;
                case 0x04 : blink_led2(4,1);
                            program(0x0C,0x01); //12456
                            break;
                case 0x05 : blink_led2(5,1);
                            program(0x0C,0x01); //12650
                            break;
                case 0x06 : blink_led2(6,1);
                            program(0x0C,0x01); //13000
                            break;
                case 0x07 : blink_led2(7,1);
                            program(0x0C,0x01); //13000
                            break;
                case 0x08 : blink_led2(8,1);
                            program(0x0C,0x01); //13200
                            break;
                default:    blink_led2(10,1);
                            break;            
            }
            break;
        case 0x0F:  // Dip Switch Configuration: X 1111 XXXX
                    //Test!
            blink_led(9,1);
            switch (frequency_cfg) {
                case 0x01 : blink_led2(1,1);
                            test_dfs(0x01);//DFS-1301 Rev. C
                            break;
                case 0x02 : blink_led2(2,1);
                            test_dfs(0x02);//DFS-1301 Rev.D 
                            break;
                case 0x03 : blink_led2(3,1);
                            break;
                case 0x04 : blink_led2(4,1);
                            test_dfs(0x04);//DFS-1201 Rev. D
                            break;
                case 0x05 : blink_led2(5,1);
                            test_dfs(0x05);//DFS-1101
                            break;
                case 0x06 : blink_led2(6,1);
                            test_dfs(0x0A);//ILCDFSL-1201
                            break;
                case 0x07 : blink_led2(7,1);
                            test_dfs(0x0B);//ILCDFSL-1301 SBX
                            break;
                case 0x08 : blink_led2(8,1);
                            test_dfs(0x0C);//ILCDFSL-1301
                            break;
                default:    blink_led2(10,1);
                            break;            
            }
            break;
        default:    
            blink_led(10,1);
            break;
            
            
    }//switch hardware
    delay_long(4);
    delay_long(4);
    while(1){
    LED2=1;    
    LED1=1;    
    }
 
    
    }//main